import "package:flutter/material.dart";
import 'dart:math';

class ColorLoader3 extends StatefulWidget {
  final double mainBall;
  final double subBall;
  final bool reverse;
  final bool onOff;
  final List<Color> color;

  ColorLoader3(
      {Key key,
      this.reverse = false,
      this.mainBall = 20.0,
      this.subBall = 5.0,
      this.onOff = false,
      this.color = const [
        Colors.amber,
        Colors.deepOrangeAccent,
        Colors.pinkAccent,
        Colors.purple,
        Colors.yellow,
        Colors.lightGreen,
        Colors.orangeAccent,
        Colors.blueAccent
      ]});

  @override
  _ColorLoader3State createState() => _ColorLoader3State();
}

class _ColorLoader3State extends State<ColorLoader3>
    with SingleTickerProviderStateMixin {
  // ignore: non_constant_identifier_names
  Animation<double> animation_rotation;

  // ignore: non_constant_identifier_names
  Animation<double> animation_mainBall_in;

  // ignore: non_constant_identifier_names
  Animation<double> animation_mainBall_out;
  AnimationController controller;

  double mainBall;
  double subBall;

  @override
  void initState() {
    super.initState();

    mainBall = widget.mainBall;
    subBall = widget.subBall;

    controller = AnimationController(
        lowerBound: 0.0,
        upperBound: 1.0,
        duration: const Duration(milliseconds: 3000),
        vsync: this);

    animation_rotation =
        Tween(begin: 0.0, end: widget.reverse ? -1.0 : 1.0).animate(
      CurvedAnimation(
        parent: controller,
        curve: Interval(0.0, 1.0, curve: Curves.linear),
      ),
    );

    animation_mainBall_in =
        Tween(begin: 1.0, end: widget.onOff ? 0.0 : 1.0).animate(
      CurvedAnimation(
        parent: controller,
        curve: Interval(0.75, 1.0, curve: Curves.elasticIn),
      ),
    );

    animation_mainBall_out =
        Tween(begin: widget.onOff ? 0.0 : 1.0, end: 1.0).animate(
      CurvedAnimation(
        parent: controller,
        curve: Interval(0.0, 0.25, curve: Curves.elasticOut),
      ),
    );

    controller.addListener(() {
      setState(() {
        if (controller.value >= 0.75 && controller.value <= 1.0)
          mainBall = widget.mainBall * animation_mainBall_in.value;
        else if (controller.value >= 0.0 && controller.value <= 0.25)
          mainBall = widget.mainBall * animation_mainBall_out.value;
      });
    });

    controller.addStatusListener((status) {
      if (status == AnimationStatus.completed) {}
    });

    controller.repeat();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 100.0, height: 100.0, //color: Colors.black12,
      child: new Center(
        child: new RotationTransition(
          turns: animation_rotation,
          child: new Container(
            //color: Colors.limeAccent,
            child: new Center(
              child: Stack(
                children: <Widget>[
                  new Transform.translate(
                    offset: Offset(0.0, 0.0),
                    child: Dot(
                      mainBall: mainBall,
                      color: Colors.black12,
                    ),
                  ),
                  new Transform.translate(
                    child: Dot(
                      mainBall: subBall,
                      color: widget.color.length < 1
                          ? Colors.amber
                          : widget.color[0],
                    ),
                    offset: Offset(
                      mainBall * cos(0.0),
                      mainBall * sin(0.0),
                    ),
                  ),
                  new Transform.translate(
                    child: Dot(
                      mainBall: subBall,
                      color: widget.color.length < 2
                          ? Colors.deepOrangeAccent
                          : widget.color[1],
                    ),
                    offset: Offset(
                      mainBall * cos(0.0 + 1 * pi / 4),
                      mainBall * sin(0.0 + 1 * pi / 4),
                    ),
                  ),
                  new Transform.translate(
                    child: Dot(
                      mainBall: subBall,
                      color: widget.color.length < 3
                          ? Colors.pinkAccent
                          : widget.color[2],
                    ),
                    offset: Offset(
                      mainBall * cos(0.0 + 2 * pi / 4),
                      mainBall * sin(0.0 + 2 * pi / 4),
                    ),
                  ),
                  new Transform.translate(
                    child: Dot(
                      mainBall: subBall,
                      color: widget.color.length < 4
                          ? Colors.purple
                          : widget.color[3],
                    ),
                    offset: Offset(
                      mainBall * cos(0.0 + 3 * pi / 4),
                      mainBall * sin(0.0 + 3 * pi / 4),
                    ),
                  ),
                  new Transform.translate(
                    child: Dot(
                      mainBall: subBall,
                      color: widget.color.length < 5
                          ? Colors.yellow
                          : widget.color[4],
                    ),
                    offset: Offset(
                      mainBall * cos(0.0 + 4 * pi / 4),
                      mainBall * sin(0.0 + 4 * pi / 4),
                    ),
                  ),
                  new Transform.translate(
                    child: Dot(
                      mainBall: subBall,
                      color: widget.color.length < 6
                          ? Colors.lightGreen
                          : widget.color[5],
                    ),
                    offset: Offset(
                      mainBall * cos(0.0 + 5 * pi / 4),
                      mainBall * sin(0.0 + 5 * pi / 4),
                    ),
                  ),
                  new Transform.translate(
                    child: Dot(
                      mainBall: subBall,
                      color: widget.color.length < 7
                          ? Colors.orangeAccent
                          : widget.color[6],
                    ),
                    offset: Offset(
                      mainBall * cos(0.0 + 6 * pi / 4),
                      mainBall * sin(0.0 + 6 * pi / 4),
                    ),
                  ),
                  new Transform.translate(
                    child: Dot(
                      mainBall: subBall,
                      color: widget.color.length < 8
                          ? Colors.blueAccent
                          : widget.color[7],
                    ),
                    offset: Offset(
                      mainBall * cos(0.0 + 7 * pi / 4),
                      mainBall * sin(0.0 + 7 * pi / 4),
                    ),
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }
}

class Dot extends StatelessWidget {
  final double mainBall;
  final Color color;

  Dot({this.mainBall, this.color});

  @override
  Widget build(BuildContext context) {
    return new Center(
      child: Container(
        width: mainBall,
        height: mainBall,
        decoration: BoxDecoration(color: color, shape: BoxShape.circle),
      ),
    );
  }
}
